close all
clear all
clc
format compact

% *** Design Process: ***
% 
% We first selected Rs, the series resistor between Vcc and the NTC.  
% If Rs is too small, we dissapate too much power in the NTC, which may 
% cause damage or cause a self-heating bias in the temperature measurement.  
% However, large values of Rs will cause very small voltages across the NTC.  
% We selected 5kOhms for the series resistor as this limits power 
% dissapation in the NTC to 12mW, safely below the limit of 50mW specified 
% in the datasheet.  
% 
% Next, we wrote three requirements for the signal conditioning circuit, 
% which will take the voltage across the NTC and make it readable by the
% ADC of the ATMega.  
% 1) operate in NTC temperature range of 15C to 200C
% 2) voltage must be in the range [0.5V, 4.5V].  Outside of this range, 
%    the assumption of ADC linearity is invalidated. 
% 3) the change in voltage for a 0.5K change in temperature must exceed 
%    the resolution of an 8-bit ADC with 5V reference (i.e., about 20mV)
% 
% Using these requirements, we chose four software-selectable gains for a 
% non-inverting amplifier circuit. Without any gain, we see that the 
% resolution requirement is only satisfied in a narrow temperature range.  
% We then calculate the minimum gain to meet the resolution specificant for 
% each temperature.  Next, we calculate the output voltage with the minimum 
% gain, seeing that it is in the required range for all the temperatures we 
% need.  This demonstrates a solution is possible if we can have numerous 
% selectable gains.  The next step is to choose gains for different 
% temperature regions that meet the requirements.  We chose four gain 
% regions and tweaked them until they met spec.  Finally, we caclulated the 
% value of the feedback resistors to achieve these gains, and then slightly 
% changed the gains to produce resistor values available for purchase.
%
% **********************

delTemp = 0.5;              % temperature resolution for the sim [K]
T = 273.15+(0:delTemp:200); % temperature to simulate [K]
TinC = T-273.15;            % convert for ease of use [degC]
R0 = 10e3;                  % nominal resistance (at T0) [Ohms]
T0 = 298.15;                % nominal temperature [K]
B = 3450;                   % B value of the NTC thermistor
Rs = 5e3;                   % the resistor between Vcc and the NTC [Ohms]
rNTC = R0*exp(B*(1./T - 1/T0));     % R(T) [Ohms]
vNTC = 5 * rNTC./(Rs+rNTC);  % voltage across the NTC [V]
LSBs = 255*(vNTC/5);                % number of LSBs for vNTC in an 8-bit ADC

req_delV = 20e-3;

% now go in reverse
% vNTC = LSBs * (5/1024);
% rNTC = Rs*(vNTC/5/gain)./(1-(vNTC/5/gain));
% T = B/(log(rNTC) - log(R0) + B/T0);

figure, plot(TinC,rNTC),  title('Resistance of the thermistor')
figure, plot(TinC,vNTC), xlabel('T [degC]'), ylabel('v_{NTC} [V]'), title('Voltage across the thermistor')

current = 5./(Rs + rNTC);   % current through the thermistor [Amps]
figure, plot(TinC,vNTC.*current), xlabel('T [degC]'), ylabel('P_{NTC} [W]'), title('Power dissapated in NTC')
dv = diff(vNTC);    % find the change in voltage for a delTemp change in T
% find the ratio between the required deltaV and the observed deltaV
% -> this is the required gain of the measuring circuit
req_gain = -req_delV./dv; 

figure, plot(TinC(1:end-1), req_gain), xlabel('T [degC]'), ylabel('Gain'), title('Minimum gain to meet the resolution spec');

figure, plot(TinC(1:end-1), -dv), hold on, plot([0 200],[req_delV req_delV],':k'), hold off
legend('Predicted','Requirement')
xlabel('T [degC]'), ylabel('\Delta v_{NTC} [V/(0.5degC)]'), title('\Delta v_{NTC} without gain')

figure, plot(TinC(1:end-1), vNTC(1:end-1).*req_gain), xlabel('T [degC]'), ylabel('v_{NTC} [V]'), title('vNTC with the minimum gain')

% assign gains for different ranges
% these values were adjusting until they met the design requirements
gain(1:110) = 1;
gain(111:185) = 2.0975;
gain(186:260) = 4.8077;
gain(261:340) = 9.9989;
gain(341:401) = 20.56;

figure, plot(TinC,gain), xlabel('T [degC]'), title('Gain assigned to different ranges');

% now multiply the raw vNTC by the assigned gain to simulate the ouput
% voltage of the amplifier
% plot, comparing to the required voltage range of 0.5 to 4.5 V
figure, plot(TinC,vNTC.*gain), hold on, plot([0 200],[4.5 4.5],':k'), plot([0 200],[0.5 0.5],':k'), hold off
xlabel('T [degC]'), ylabel('v_{out} [V]')
title('Output voltage (including gain)')

% now find the delta V with the output voltage
% plot, comparing to the required delta V of 20mV per 0.5degC
figure, plot(TinC(1:end-1), -diff(gain.*vNTC)), hold on, plot([0 200],[req_delV req_delV],':k'), hold off
xlabel('T [degC]'), ylabel('\Delta v_{out} [V/(0.5degC)]')
title('\Delta v_{out} (including gain)'), ylim([0 0.1])

% we used this formula to choose the feedback resistors
% 22.5 ohms is the output impendance of the ATmega644 GPIO port, which is
% effectively added to the feedback resistor

fb = 20e3./(gain-1) - 22.5;

fb_true = [100e3, 18.2e3, 5.23e3, 2.2e3, 1e3];

true_gain = 1 + 20e3./(22.5 + fb_true)


